/*
 * LEAPS - Low Energy Accurate Positioning System.
 *
 * Copyright (c) 2016-2017, LEAPS. All rights reserved.
 */

package com.decawave.argomanager.argoapi.ble.connection;

/**
 * Poll command stored in Firmware Update Poll Characteristic.
 * Always generated by network node (GATT server), consumed by mobile app (GATT client).
 * Consume is done via notifications.
 */
public interface FwPollCommand {

    /**
     * Type of command.
     */
    enum Type {
        FW_UPLOAD_REFUSED,
        FW_BUFFER_REQUEST,
        FW_UPLOAD_COMPLETE,
        FW_SAVE_FAILED
    }

    Type getType();

    void accept(PollPayloadVisitor visitor);

    ///////////////////////////////////////////////////////////////////////////////////////////
    //
    // existing commands
    //

    class UploadRefused implements FwPollCommand {
        public final int errorCode;

        public UploadRefused(int errorCode) {
            this.errorCode = errorCode;
        }

        @Override
        public Type getType() {
            return Type.FW_UPLOAD_REFUSED;
        }

        @Override
        public void accept(PollPayloadVisitor visitor) {
            visitor.visit(this);
        }

        @Override
        public String toString() {
            return "UploadRefused{" +
                    "errorCode=" + errorCode +
                    '}';
        }
    }

    class BufferRequest implements FwPollCommand {
        public final int offset;
        public final int size;

        public BufferRequest(int offset, int size) {
            this.offset = offset;
            this.size = size;
        }

        @Override
        public Type getType() {
            return Type.FW_BUFFER_REQUEST;
        }

        @Override
        public void accept(PollPayloadVisitor visitor) {
            visitor.visit(this);
        }

        @Override
        public String toString() {
            return "BufferRequest{" +
                    "offset=" + offset +
                    ", size=" + size +
                    '}';
        }
    }

    class UploadComplete implements FwPollCommand {
        @Override
        public Type getType() {
            return Type.FW_UPLOAD_COMPLETE;
        }

        @Override
        public void accept(PollPayloadVisitor visitor) {
            visitor.visit(this);
        }

        @Override
        public String toString() {
            return "UploadComplete{}";
        }
    }

    class SaveFailed implements FwPollCommand {
        public final int errorCode;

        public SaveFailed(int errorCode) {
            this.errorCode = errorCode;
        }

        @Override
        public Type getType() {
            return Type.FW_SAVE_FAILED;
        }

        @Override
        public void accept(PollPayloadVisitor visitor) {
            visitor.visit(this);
        }

        @Override
        public String toString() {
            return "SaveFailed{" +
                    "errorCode=" + errorCode +
                    '}';
        }
    }


}
